@@ -1,4 +1,5 @@
-from typing import Union
+from typing import Iterable, Union, Callable
+from string import printable
 
 
 class SerialzeValueError(ValueError):
@@ -10,6 +11,41 @@ class ref:
         self.obj = obj
 
 
+class S:
+    'See https://github.com/frankli0324/libphpserialize/wiki/PHP-String-and-Binary-Data'
+    def __init__(self, s: Union[str, bytes, Iterable[int]],
+                 encode_chars: Union[str, bytes, Callable[[int], bool]] = None,
+                 encode_all=False,
+                 format='02x'):
+        if type(encode_chars) == str:
+            encode_chars = encode_chars.encode()
+        if type(encode_chars) == bytes:
+            self.encode_chars = lambda x: x in encode_chars
+        elif not encode_chars:
+            self.encode_chars = lambda x: x not in bytes(printable, 'utf-8')
+        else:
+            self.encode_chars = encode_chars
+        self.encode_all = encode_all
+        self.format = format
+        if type(s) == str:
+            s = s.encode('utf-8')
+        self.s = s
+
+    def __getitem__(self, item):
+        if type(item) != int or item < 0 or item > 256:
+            raise SerialzeValueError()
+        if self.encode_all or self.encode_chars(item) or item == 92:
+            return '\\' + format(item, self.format)
+        else:
+            return chr(item)
+
+    def encode(self):
+        return ''.join(self[c] for c in self.s)
+
+    def __str__(self):
+        return f'S:{len(self.s)}:"{self.encode()}";'
+
+
 class __empty__:
     __namespace__ = None
     pass
@@ -88,7 +124,8 @@ def _handle_number(num: Union[int, float]):
 
 
 _handlers = {
-    str: lambda x: f's:{len(x.encode())}:"{x}";',
+    str: lambda x: f's:{len(x.encode("utf-8"))}:"{x}";',
+    S: lambda x: str(x),
     int: _handle_number,
     float: _handle_number,
     list: _handle_array,
@@ -105,41 +142,13 @@ def register_handler(type, handler):
 def _serialize(obj):
     if type(obj) in _handlers:
         return _handlers[type(obj)](obj)
-    # basic types are not tracked
-    # see ext/standard/var.c, around line 610
-    # if (!is_ref && Z_TYPE_P(var) != IS_OBJECT) {
-	#     return 0;
-	# }
-    # PHP does recognize basic type implicit references though
-    # php > var_dump(unserialize("a:2:{i:0;i:1;i:1;R:2;}"));
-    # array(2) {
-    #   [0]=>
-    #   &int(1)
-    #   [1]=>
-    #   &int(1)
-    # }
-    # php > var_dump(unserialize("a:2:{i:0;i:1;i:1;r:2;}"));
-    # PHP Notice:  unserialize(): Error at offset 21 of 22 bytes in php shell code on line 1
-    # Notice: unserialize(): Error at offset 21 of 22 bytes in php shell code on line 1
-    # bool(false)
+    # wiki/Handling-Reference-Types#tracking-objects
     if (t := track.get(obj)):
         return f'r:{t};'
 
     if type(obj) == ref:
         if not (i := track.get(obj.obj)):
-            # PHP would dereference and serialize the value normally
-            # since people would want references explicitly when writing Python scripts,
-            # an error is raised here.
-            # though that brings up the problem of ordering, take the following example:
-            # ```php
-            # php > class Obj{};
-            # php > $a = new Obj();
-            # php > $b = &$a;
-            # php > echo serialize(array($a, $b));
-            # a:2:{i:0;O:3:"Obj":0:{}i:1;r:2;}
-            # php > echo serialize(array($b, $a)); # this would raise an error in libphpserialize
-            # a:2:{i:0;O:3:"Obj":0:{}i:1;r:2;}
-            # ```
+            # wiki/Handling-Reference-Types#serialzevalueerror-invalid-reference
             raise SerialzeValueError("Invalid Reference")
         return f'R:{i};'
     track.put(obj)
@@ -151,4 +160,4 @@ def serialize(obj):
     return _serialize(obj)
 
 
-__all__ = ['serialize', 'register_handler', 'ref', 'SerialzeValueError', ]
+__all__ = ['serialize', 'register_handler', 'ref', 'S', 'SerialzeValueError', ]
